#import "TableDemoViewController.h"

@implementation TableDemoViewController

- (id)init {
	self = [super init];
	if (self != nil) {
		/* ファイルのリストを構築 */
		[self reload];
		
		/* ナビゲーションバーのボタンの初期化 */
		self.navigationItem.rightBarButtonItem  = [[[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemEdit
																								 target:self
																								 action:@selector(startEditing)] autorelease];
		self.navigationItem.leftBarButtonItem = [[[UIBarButtonItem alloc] initWithTitle:@"再読込" 
																				  style:UIBarButtonItemStylePlain
																				 target:self 
																				 action:@selector(reload)] autorelease];	
	}
	return self;
}

- (void)startEditing {
	[self.tableView setEditing:YES animated:YES];
	
	self.navigationItem.rightBarButtonItem = [[[UIBarButtonItem alloc ] initWithBarButtonSystemItem:UIBarButtonSystemItemDone
																							 target:self
																							 action:@selector(stopEditing)] autorelease];	
}

- (void)stopEditing {
	[self.tableView setEditing:NO animated:YES];
	
	self.navigationItem.rightBarButtonItem = [[[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemEdit
																							target:self
																							action:@selector(startEditing)] autorelease];
}

- (void)reload {
	NSDirectoryEnumerator *dirEnum;
	NSString *file;
	
	fileList = [[NSMutableArray alloc] init];
	dirEnum = [[NSFileManager defaultManager] enumeratorAtPath:NSHomeDirectory()];
	while ((file = [dirEnum nextObject])) {
		[fileList addObject:file];
	}
	
	[self.tableView reloadData];
}

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
	return 1;
}


- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
	return [fileList count];
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
	NSString *cellIdentifier = [fileList objectAtIndex:[indexPath indexAtPosition:1]];
	
	UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:cellIdentifier];
	if (cell == nil) {
		cell = [[[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:cellIdentifier] autorelease];
		cell.textLabel.text = cellIdentifier;
		
		UIFont *font = [UIFont fontWithName:@"Courier" size:12.0];
		cell.textLabel.font = font;
    }
	
	return cell;
}

- (void)tableView:(UITableView *)tableView commitEditingStyle:(UITableViewCellEditingStyle)editingStyle forRowAtIndexPath:(NSIndexPath *)indexPath {
	if (editingStyle == UITableViewCellEditingStyleDelete) {
		/* データソースからセルを消去 */
		UITableViewCell *cell = [self.tableView cellForRowAtIndexPath:indexPath];
		for(int i = 0; i < [fileList count]; i++) {
			if ([cell.textLabel.text isEqualToString:[fileList objectAtIndex:i]]) {
				[fileList removeObjectAtIndex:i];
			}
		}
		
		/* テーブルからセルを消去 */
		NSMutableArray *array = [[NSMutableArray alloc] init];
		[array addObject:indexPath];
		[self.tableView deleteRowsAtIndexPaths:array withRowAnimation:UITableViewRowAnimationTop];
	} 
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
	[tableView deselectRowAtIndexPath:indexPath animated:YES];
	UITableViewCell *cell = [tableView cellForRowAtIndexPath:indexPath];
	
	UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"ファイルの選択" 
													message:[NSString stringWithFormat:@"ファイル'%@'が選択されました", cell.textLabel.text]
												   delegate:nil
										  cancelButtonTitle:nil
										  otherButtonTitles:@"OK", nil];
	
	[alert show];
	[alert release];
}


- (void)loadView {
	[super loadView];
}


- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
	return YES; 
}


- (void)didReceiveMemoryWarning {
	[super didReceiveMemoryWarning];
}


- (void)dealloc {
  [fileList release];
  [super dealloc];
}

@end
