#import <CoreGraphics/CoreGraphics.h>
#import "AVMeterView.h"

@implementation AVMeterView
@synthesize player;
@synthesize meterAveragePowerImage;
@synthesize meterPeakPowerImage;
@synthesize meterSpacing;
@synthesize meterHorizontalBorder;
@synthesize meterVerticalBorder;

- (id)initWithFrame:(CGRect)frame {
	self = [super initWithFrame:frame];
	if (self != nil) {
		self.backgroundColor = [UIColor blackColor];
		player = nil;
		updating = NO;
		
		meterAveragePowerImage = nil;
		meterPeakPowerImage = nil;
		
		meterSpacing = 40.0;
		meterHorizontalBorder = 20.0;
		meterVerticalBorder = 20.0;
		
		for(int i = 0; i < 2; i ++) {
			cachedPeakPowerForChannel[i] = 0.0;
			cachedAveragePowerForChannel[i] = 0.0;
		}
		
		/* メータの位置を計算 */
		avgMeterFrame[0] = CGRectMake(meterHorizontalBorder,
									  meterVerticalBorder,
									  (frame.size.width / 2) - (meterSpacing/2) - meterHorizontalBorder,
									  (frame.size.height) - meterVerticalBorder);
		avgMeterFrame[1] = CGRectMake(meterHorizontalBorder + (frame.size.width / 2),
									  meterVerticalBorder,
									  (frame.size.width / 2) - (meterSpacing/2) - meterHorizontalBorder,
									  frame.size.height - meterVerticalBorder);
		peakMeterFrame[0] = CGRectMake(avgMeterFrame[0].origin.x, avgMeterFrame[0].origin.y,
									   avgMeterFrame[0].size.width, meterPeakPowerImage.size.height);
		peakMeterFrame[1] = CGRectMake(avgMeterFrame[1].origin.x, avgMeterFrame[1].origin.y,
									   avgMeterFrame[1].size.width, meterPeakPowerImage.size.height);
	}
	return self;
}

- (void)drawRect:(CGRect)rect {
	float averagePowerForChannel[2], peakPowerForChannel[2];
	BOOL renderedBottom;
	
	if (player.numberOfChannels < 1) {
		return;
	}
	
	/* メータの値を読む */
	if (!player || player.meteringEnabled == NO) {
		averagePowerForChannel[0] = averagePowerForChannel[1] = 0.0;
		peakPowerForChannel[0] = cachedPeakPowerForChannel[0];
		peakPowerForChannel[1] = cachedPeakPowerForChannel[1];
	} else {
		int channels = player.numberOfChannels;
		if (channels > 2)
			channels = 2;
		for(int i = 0; i < channels; i++) {
			float db;
			[player updateMeters];
			db = [player peakPowerForChannel:i];
			peakPowerForChannel[i] = (50.0 + db) / 50.0;
      NSLog(@"db=%f", db);
			db = [player averagePowerForChannel:i];
			averagePowerForChannel[i] = (50.0 + db) / 50.0;
			NSLog(@"Power for channel %d: %f DB %f Peak: %f\n", i, averagePowerForChannel[i], db, peakPowerForChannel[i]);
		}
		if (channels == 1) {
			peakPowerForChannel[1] = peakPowerForChannel[0];
			averagePowerForChannel[1] = averagePowerForChannel[0];
		}
	}
	
	/* 新レベルにするか、メータを減らす */
	renderedBottom = YES;
	for(int i = 0; i < 2; i++) {
		if (averagePowerForChannel[i] > cachedAveragePowerForChannel[i]) {
			cachedAveragePowerForChannel[i] = averagePowerForChannel[i];
		} 
		cachedAveragePowerForChannel[i] -= 0.05;
		if (cachedAveragePowerForChannel[i] < 0) {
			cachedAveragePowerForChannel[i] = 0;
		}
		
		if (peakPowerForChannel[i] > cachedPeakPowerForChannel[i]) {
			cachedPeakPowerForChannel[i] = peakPowerForChannel[i];
		} 
		cachedPeakPowerForChannel[i] -= 0.01;
		if (cachedPeakPowerForChannel[i] < 0.0)
			cachedPeakPowerForChannel[i] = 0.0;
		
		if (cachedPeakPowerForChannel[i] != 0.0 || cachedAveragePowerForChannel[i] != 0.0) {
			renderedBottom = NO;
		}
		
		if (meterAveragePowerImage) {
			[meterAveragePowerImage drawAsPatternInRect:
			 CGRectMake(avgMeterFrame[i].origin.x,
					    avgMeterFrame[i].origin.y + (avgMeterFrame[i].size.height-(avgMeterFrame[i].size.height * cachedAveragePowerForChannel[i])),
					    avgMeterFrame[i].size.width,
					    avgMeterFrame[i].size.height - (avgMeterFrame[i].size.height-(avgMeterFrame[i].size.height * cachedAveragePowerForChannel[i])))];
		}
		
		if (meterPeakPowerImage) {
			[meterPeakPowerImage drawInRect:
			 CGRectMake(peakMeterFrame[i].origin.x,
						peakMeterFrame[i].origin.y + (avgMeterFrame[i].size.height-(avgMeterFrame[i].size.height * cachedPeakPowerForChannel[i])),
						peakMeterFrame[i].size.width,
						meterPeakPowerImage.size.height)];
		}
	}
	
	if (updating == YES || renderedBottom == NO) {
		[NSTimer scheduledTimerWithTimeInterval:0.01
										 target:self
									   selector:@selector(handleTimer:)
									   userInfo:nil
										repeats:NO];
	}
}

- (void)handleTimer:(NSTimer *)timer {
	[self setNeedsDisplay];
}

- (void)startUpdating {
	updating = YES;
	player.meteringEnabled = YES;
	[self setNeedsDisplay];
}

- (void)stopUpdating {
	updating = NO;
	player.meteringEnabled = NO;
	[self setNeedsDisplay];
}

- (void)dealloc {
	[player release];
	[meterPeakPowerImage release];
	[meterAveragePowerImage release];
	[super dealloc];
}

@end
