#import "AVMeterViewController.h"

@implementation AVMeterViewController

- (id)init {
	self = [super init];
	if (self != nil) {
		NSError *err;
		
		player = [[AVAudioPlayer alloc] initWithContentsOfURL:
				  [NSURL fileURLWithPath:[[NSBundle mainBundle] pathForResource:@"sample" ofType:@"mp3"]]
														error: &err];
		
		if (err)
			NSLog(@"Failed to initialize AVAudioPlayer: %@\n", err);
		
		[self setNavProperties];
		player.delegate = self;
		[player prepareToPlay];
	}
	return self;
}

- (void)loadView {
	[super loadView];
	
	CGRect frame = self.view.frame;	
	meterView = [[AVMeterView alloc] initWithFrame:frame];
	meterView.player = player;
	
	
	UIImage *peak = [[[UIImage alloc] initWithContentsOfFile:[[NSBundle mainBundle] pathForResource:@"peakMeterImage" ofType:@"png" inDirectory:@"/"]] autorelease];
	UIImage *avg = [[[UIImage alloc] initWithContentsOfFile:[[NSBundle mainBundle] pathForResource: @"avgMeterImage" ofType:@"png" inDirectory:@"/"]] autorelease];
	meterView.meterPeakPowerImage = peak;
	meterView.meterAveragePowerImage = avg;
	
	self.view = meterView;
}

- (void)setNavProperties {
	navButton = [[UIBarButtonItem alloc] initWithTitle: (player.playing == YES) ? @"Stop" : @"Play"
												 style:UIBarButtonItemStylePlain
												target:self 
												action:@selector(navButtonWasPressed)];
	self.navigationItem.rightBarButtonItem = navButton;	
	
	if (player.playing == YES)
		self.title = @"Playing";
	else
		self.title = @"Stopped";
}

- (void)audioPlayerDidFinishPlaying:(AVAudioPlayer *)player successfully:(BOOL)flag {
	[meterView stopUpdating];
	[self setNavProperties];
}

- (void)navButtonWasPressed {
	if (player.playing == YES) {
		[player stop];
		[meterView stopUpdating];
	} else {
		[player play];
		[meterView startUpdating];
	}
	[self setNavProperties];	
}

- (void)dealloc {
	[navButton release];
	[meterView release];
	[super dealloc];
}

@end
