
#import "RecorderViewController.h"
#include <AudioToolbox/AudioToolbox.h>

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <sys/stat.h>
#include <sys/select.h>

#define AUDIO_BUFFERS 3

typedef struct AQCallbackStruct {
    AudioStreamBasicDescription mDataFormat;
    AudioQueueRef queue;
    AudioQueueBufferRef mBuffers[AUDIO_BUFFERS];
    AudioFileID outputFile;
    
    unsigned long frameSize;
    long long recPtr;
    int run;
} AQCallbackStruct;

static void AQInputCallback (
                             void                                 *aqr,
                             AudioQueueRef                        inQ,
                             AudioQueueBufferRef                  inQB,
                             const AudioTimeStamp                 *timestamp,
                             unsigned long                        frameSize,
                             const AudioStreamPacketDescription   *mDataFormat)
{
    AQCallbackStruct *aqc = (AQCallbackStruct *) aqr;
    
    /* Write data to file */
    if (AudioFileWritePackets (aqc->outputFile, false, inQB->mAudioDataByteSize,
                               mDataFormat, aqc->recPtr, &frameSize, inQB->mAudioData) == noErr)
    {
        aqc->recPtr += frameSize;
    }
    
    /* Don't re-queue the sound buffers if we're supposed to stop recording */
    if (!aqc->run)
        return;
    
    AudioQueueEnqueueBuffer (aqc->queue, inQB, 0, NULL);
}


static void SoundFinished(SystemSoundID sample, void *info){
    AudioServicesDisposeSystemSoundID(sample);
}

@implementation RecorderViewController

- (void)play:(id)sender{
    AQCallbackStruct aqc;
    AudioFileTypeID fileFormat;
    CFURLRef filename;
    struct timeval tv;
    int i;
    
    aqc.mDataFormat.mFormatID = kAudioFormatLinearPCM;
    aqc.mDataFormat.mSampleRate = 44100.0;
    aqc.mDataFormat.mChannelsPerFrame = 2;
    aqc.mDataFormat.mBitsPerChannel = 16;
    aqc.mDataFormat.mBytesPerPacket =
    aqc.mDataFormat.mBytesPerFrame =
    aqc.mDataFormat.mChannelsPerFrame * sizeof (short int);
    aqc.mDataFormat.mFramesPerPacket = 1;
    aqc.mDataFormat.mFormatFlags =
    kLinearPCMFormatFlagIsBigEndian
    | kLinearPCMFormatFlagIsSignedInteger
    | kLinearPCMFormatFlagIsPacked;
    aqc.frameSize = 735;
    
    AudioQueueNewInput (&aqc.mDataFormat, AQInputCallback, &aqc, NULL,
                        kCFRunLoopCommonModes, 0, &aqc.queue);
    
    /* Create output file */
    
    fileFormat = kAudioFileAIFFType;
    NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
    NSString *documentsDirectory = [paths objectAtIndex:0];
    filename = (CFURLRef)[NSURL fileURLWithPath:[NSString stringWithFormat:@"%@/test.aif", documentsDirectory]];
    
    AudioFileCreateWithURL (
                            filename,
                            fileFormat,
                            &aqc.mDataFormat,
                            kAudioFileFlags_EraseFile,
                            &aqc.outputFile
                            );
    
    /* Initialize the recording buffers */
    
    for (i=0; i<AUDIO_BUFFERS; i++) {
        AudioQueueAllocateBuffer (aqc.queue, aqc.frameSize, &aqc.mBuffers[i]);
        AudioQueueEnqueueBuffer (aqc.queue, aqc.mBuffers[i], 0, NULL);
    }
    
    aqc.recPtr = 0;
    aqc.run = 1;
    
    AudioQueueStart (aqc.queue, NULL);
    
    /* Hang around for a while while the recording takes place */
    
    tv.tv_sec = 5.0;
    tv.tv_usec = 0;
    select(0, NULL, NULL, NULL, &tv);
    
    /* Shut down recording */
    
    AudioQueueStop (aqc.queue, true);
    aqc.run = 0;
    
    AudioQueueDispose (aqc.queue, true);
    AudioFileClose (aqc.outputFile);
    
    SystemSoundID soundID;
    AudioServicesCreateSystemSoundID(filename, &soundID);
    AudioServicesAddSystemSoundCompletion(soundID, NULL, NULL, SoundFinished, NULL);
    AudioServicesPlaySystemSound(soundID);
}

- (void)viewDidLoad{
    button = [[UIButton buttonWithType:UIButtonTypeRoundedRect] retain];
    [button setTitle:@"録音" forState:UIControlStateNormal];
    [button addTarget:self action:@selector(play:) forControlEvents:UIControlEventTouchUpInside];
    button.frame = CGRectMake(0, 0, 320, 40);
    [self.view addSubview:button];
}

- (void)dealloc {
    [button release];
    [super dealloc];
}


@end
