#import "TableDemoViewController.h"

@implementation TableDemoViewController

- (id)init {
    self = [super init];
    
    if (self != nil) {
        
        /* ファイルのリストを作成 */
        [self reload];
        
        /* ナビゲーションバーボタンを初期化 */
        
        self.navigationItem.rightBarButtonItem = 
            [[[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemEdit target:self action:@selector(startEditing)] autorelease];
        
        self.navigationItem.leftBarButtonItem = 
            [[[UIBarButtonItem alloc] initWithTitle:@"再読み込み" style:UIBarButtonItemStylePlain target:self action:@selector(reload)] autorelease];    
    }
    
    return self;
}

- (NSArray *)sectionIndexTitlesForTableView:(UITableView *)tableView {
    
    return sectionTitles;
}

- (void) startEditing {
    [self.tableView setEditing:YES animated:YES];
    
    self.navigationItem.rightBarButtonItem = 
        [[[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemDone target:self action:@selector(stopEditing)] autorelease];    
}

- (void) stopEditing {
    [self.tableView setEditing:NO animated:YES];
    
    self.navigationItem.rightBarButtonItem  = 
        [[[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemEdit target:self action:@selector(startEditing)] autorelease];
}

- (void) reload {
    NSDirectoryEnumerator *dirEnum;
    NSString *file;
    
	fileList = [[NSMutableArray alloc] init];
	sectionTitles = [[NSMutableArray alloc] init];
    dirEnum = [[NSFileManager defaultManager] enumeratorAtPath:[NSString stringWithFormat:@"%@/TableDemo.app/", NSHomeDirectory()]];
    while ((file = [dirEnum nextObject])) {
		NSString *initialChar = [file substringToIndex:1];
		if([@"0123456789" rangeOfString:initialChar].location != NSNotFound){
			initialChar = @"#";
		}
		if(![sectionTitles containsObject:initialChar]){
			[sectionTitles addObject:initialChar];
			[fileList addObject:[NSMutableArray arrayWithCapacity:10]];
		}
		int index = [sectionTitles indexOfObject:initialChar];
		[[fileList objectAtIndex:index] addObject:file];
    }
    [self.tableView reloadData];
}

- (NSString *)tableView:(UITableView *)tableView titleForHeaderInSection:(NSInteger)section { 
    return [sectionTitles objectAtIndex:section];
}

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
    return sectionTitles.count;
}


- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    
    return ((NSArray *)[fileList objectAtIndex:section]).count;
}

- (NSInteger)tableView:(UITableView *)tableView sectionForSectionIndexTitle:(NSString *)title atIndex:(NSInteger) index {
    int i = 0;
    for (NSString * sectionTitle in sectionTitles) {
        if ([sectionTitle isEqualToString:title]) {
            [tableView scrollToRowAtIndexPath:[NSIndexPath indexPathForRow:0 inSection:i]
                atScrollPosition:UITableViewScrollPositionTop animated:YES];
            return i;
        }
        i++;
    }
    return -1;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    
    NSString *CellIdentifier = 
        [[fileList objectAtIndex:[indexPath indexAtPosition:0]] objectAtIndex:[indexPath indexAtPosition:1]];
    
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier];
    if (cell == nil) {
        cell = [[[UITableViewCell alloc] initWithFrame:CGRectZero reuseIdentifier:CellIdentifier] autorelease];
        cell.textLabel.text = CellIdentifier;
        
        UIFont *font = [UIFont fontWithName:@"Courier" size:12.0];
        cell.textLabel.font = font;
    }
    
    return cell;
}

- (void)tableView:(UITableView *)tableView 
    commitEditingStyle:(UITableViewCellEditingStyle)editingStyle 
    forRowAtIndexPath:(NSIndexPath *)indexPath { 
    if (editingStyle == UITableViewCellEditingStyleDelete) {
        
        /* データソースからセルを消去 */
        
        UITableViewCell *cell = [self.tableView cellForRowAtIndexPath:indexPath];
        for(int i = 0; i < [[fileList objectAtIndex:[indexPath indexAtPosition:0]] count]; i++) {
            if ([cell.textLabel.text isEqualToString:[[fileList objectAtIndex:[indexPath indexAtPosition:0]] objectAtIndex:i]]) {
                [[fileList objectAtIndex:[indexPath indexAtPosition:0]] removeObjectAtIndex:i];
            }
        }
        
        /* テーブルからセルを消去 */
        
        NSMutableArray *array = [[NSMutableArray alloc] init];
        [array addObject:indexPath];
        [self.tableView deleteRowsAtIndexPaths:array withRowAnimation:UITableViewRowAnimationTop];
        
    } 
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    
    UITableViewCell *cell = [self.tableView cellForRowAtIndexPath:indexPath];
    
    UIAlertView *alert = 
        [[UIAlertView alloc] initWithTitle:@"選択されたファイル" 
            message:[NSString stringWithFormat:@"ファイル'%@'が選択されました。", cell.textLabel.text]
            delegate:nil cancelButtonTitle:nil otherButtonTitles:@"OK", nil];
    
    [alert show];
}


- (void)loadView {
    [super loadView];
}


- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
    return YES; 
}


- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
}


- (void)dealloc {
	[fileList release];
	[sectionTitles release];
    [super dealloc];
}


@end
