#import "CovertFlowViewController.h"
#import <objc/runtime.h>

@implementation CFView

- (id) initWithFrame:(CGRect)frame covers:(NSMutableArray *)covers {
	self = [super initWithFrame:frame];
	
	if (self != nil) {
		_covers = covers;
		selectedCover = 0;
		
		self.showsVerticalScrollIndicator = YES;
		self.showsHorizontalScrollIndicator = NO;
		self.delegate = self;
		self.scrollsToTop = NO;
		self.bouncesZoom = NO;
		self.bounces = NO;
		
		cfIntLayer = [[CAScrollLayer alloc] init];
		cfIntLayer.bounds = CGRectMake(0.0, 0.0, frame.size.width, frame.size.height + COVER_WIDTH_HEIGHT);
		cfIntLayer.position = CGPointMake(160.0, 304.0);
		cfIntLayer.frame = frame;
		
		for(int i = 0; i < [_covers count]; i++) {
			NSLog(@"レイヤーcfIntLayerを初期化中 %d\n", i);
			UIImageView *background = [[[UIImageView alloc] initWithImage:[_covers objectAtIndex:i]] autorelease];
			background.frame = CGRectMake(0.0, 0.0, COVER_WIDTH_HEIGHT, COVER_WIDTH_HEIGHT);
			[cfIntLayer addSublayer:background.layer];
		}
		
		self.contentSize = CGSizeMake(320.0, ((frame.size.height) + (SCROLL_PIXELS * ([_covers count] - 1))));
		
		[self.layer addSublayer:cfIntLayer];
		[self layoutLayer:cfIntLayer];
	}
	
	return self;
}

- (void)scrollViewDidScroll:(UIScrollView *)scrollView {
	selectedCover = (int)roundf((self.contentOffset.y / SCROLL_PIXELS));
	if (selectedCover > [_covers count] - 1) {
		selectedCover = [_covers count] - 1;
	}
	[self layoutLayer: cfIntLayer];
}

- (void)setSelectedCover:(int)index {
	
	if (index != selectedCover) {
		selectedCover = index;
		[self layoutLayer:cfIntLayer];
		self.contentOffset = CGPointMake(self.contentOffset.x, selectedCover * SCROLL_PIXELS);
	}
}

- (int)getSelectedCover {
	return selectedCover;
}

- (void)layoutLayer:(CAScrollLayer *)layer{
	CALayer *sublayer;
	NSArray *array;
	size_t i, count;
	CGRect rect, cfImageRect;
	CGSize cellSize, spacing, margin;
	CGSize size;
	CATransform3D leftTransform, rightTransform, sublayerTransform;
	float zCenterPosition, zSidePosition;
	float sideSpacingFactor, rowScaleFactor;
	float angle = 1.39;
	int x;
	
	size = [layer bounds].size;
	
	zCenterPosition = 60;      /* 選択されたカバーのz軸位置 */
	zSidePosition = 0;         /* その他のカバーのz軸初期位置 */
	sideSpacingFactor = 0.85;  /* スライドカバー同士の距離 */
	rowScaleFactor = 0.55;     /* メインカバーとその横のカバーの距離 */
	
	leftTransform = CATransform3DMakeRotation(angle, -1, 0, 0);
	rightTransform = CATransform3DMakeRotation(-angle, -1, 0, 0);
	
	margin = CGSizeMake(5.0, 5.0);
	spacing = CGSizeMake(5.0, 5.0);
	cellSize = CGSizeMake(COVER_WIDTH_HEIGHT, COVER_WIDTH_HEIGHT);
	
	margin.width += (size.width - cellSize.width * [_covers count]
					 -  spacing.width * ([_covers count] - 1)) * 0.5;
	margin.width = floor(margin.width);
	
	/* カバーの配列の生成 */
	array = [layer sublayers];
	count = [array count];
	sublayerTransform = CATransform3DIdentity;
	
	/* 奥行きを設定 */
	sublayerTransform.m34 = -0.006;
	
	/* すべてのアニメーションが同時に始まるようにCATransactionを開始 */    
	[CATransaction begin];
	[CATransaction setValue:[NSNumber numberWithFloat:0.3f] forKey:@"animationDuration"];
	
	for (i = 0; i < count; i++) {
		sublayer = [array objectAtIndex:i];
		x = i;
		
		rect.size = cellSize;
		rect.origin = CGPointZero;
		cfImageRect = rect;
		
		/* ベース位置 */
		rect.origin.x = size.width / 2 - cellSize.width / 2;
		rect.origin.y = margin.height + x * (cellSize.height + spacing.height);
		
		[[sublayer superlayer] setSublayerTransform:sublayerTransform];
		
		if (x < selectedCover) { /* 左（上）側 */
			rect.origin.y += cellSize.height * sideSpacingFactor
				* (float) (selectedCover - x - rowScaleFactor);
			sublayer.zPosition = zSidePosition - 2.0 * (selectedCover - x);
			sublayer.transform = leftTransform;
		}
		else if (x > selectedCover) {  /* 右（下）側 */
			rect.origin.y -= cellSize.height * sideSpacingFactor
				* (float) (x - selectedCover - rowScaleFactor);
			sublayer.zPosition = zSidePosition - 2.0 * (x - selectedCover);
			sublayer.transform = rightTransform;
		}
		else {                    /* 選択されたカバー */
			sublayer.transform = CATransform3DIdentity;
			sublayer.zPosition = zCenterPosition;
			
			/* スクロールレイヤーの中央位置 */
			[layer scrollToPoint:CGPointMake(0, rect.origin.y
											 - (([ layer bounds ].size.height - cellSize.width)/2.0))];
			
			/* ビューの中央にあるスクロールレイヤーの位置 */
			layer.position = CGPointMake(160.0f, 240.0f + (selectedCover * SCROLL_PIXELS));
        }
		[sublayer setFrame:rect];
		
    }
	[CATransaction commit];
}

@end

@implementation CovertFlowViewController

- (id)init {
	self = [super init];
	if (self != nil) {
		covers = [[NSMutableArray alloc]init];
		
		for(int i = 1; i < 6; i++) {
			NSLog(@"デモイメージをロード中（%d）\n", i);
			UIImage *image = [[UIImage alloc] initWithData:[NSData dataWithContentsOfURL:
															[NSURL URLWithString:
															 [NSString stringWithFormat:@"http://www.zdziarski.com/demo/%d.png", i]]]];
			[covers addObject:image];
		}
	}
	return self;
}

- (void)loadView {
	[super loadView];
	
	covertFlowView = [[CFView alloc] initWithFrame:self.view.frame covers:covers];
	covertFlowView.selectedCover = 2;
	
	self.view = covertFlowView;	 
}


- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
	return (interfaceOrientation == UIInterfaceOrientationPortrait);
}


- (void)didReceiveMemoryWarning {
	[super didReceiveMemoryWarning];
}


- (void)dealloc {
	[covertFlowView release];
	[super dealloc];
}

@end
